{% extends 'admin/change_form.html' %}
{% load i18n %}
{% block object-tools-items %}{{ block.super }}
<li><a href="{% url qbe_form original.pk %}" class="runlink">{% trans "Edit" %}</a></li>
<li><a href="{% url admin:savedqueries_savedquery_run original.pk %}" class="runlink">{% trans "Run" %}</a></li>
{% endblock %}
